import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.EOFException;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;
import java.util.Vector;

/*
 * psi: spherical orientation angle 1
 * phi: spherical orientation angle 2
 * rho: sphere radius
 * theta: line orientation in tangential plane 
 */
public class HoughTransformation {
	
	private final int[][][] data; // 3-D data set, [z][y][x]
	private final int dataSizeX;
	private final int dataSizeY;
	private final int dataSizeZ;
	private final int dataCenterX;
	private final int dataCenterY;
	private final int dataCenterZ;
	
	private int[][][][] accumulator; // 4-D accumulator array, [psi][phi][rho][theta]
	private int[][][][] accumulatorBackmapped; // 4-D accumulator array, [psi][phi][rho][theta]
	private final int accumulatorSizePsi;
	private final int accumulatorSizePhi;
	private final int accumulatorSizeRho;
	private final int accumulatorSizeOmega;
	private final int accumulatorCenterRho;
	
	private final double psiMin = 0;
	private final double psiMax = Math.PI;
	private final double phiMin = 0;
	private final double phiMax = Math.PI;
	private final double rhoMin; // image dependent
	private final double rhoMax; // image dependent
	private final double omegaMin = 0;
	private final double omegaMax = Math.PI;
	private int[][] roi;
	
	private final double dPsi; // psi angle increment
	private final double dPhi; // phi angle increment
	private final double dRho; // rho discretization
	private final double dOmega; // theta discretization
	private final int nPsi;
	private final int nPhi;
	private final int nRho;
	private final int nOmega;

	
	public HoughTransformation(int[][][] data, int nPsi, int nPhi, int nRho, int nOmega, int[][] roi) {
		this.data = data;
		this.dataSizeY = data[0][0].length;
		this.dataSizeX = data[0].length;
		this.dataSizeZ = data.length;
		this.dataCenterY = (int)(this.dataSizeX/2.0);
		this.dataCenterX = (int)(this.dataSizeY/2.0);
		this.dataCenterZ = (int)(this.dataSizeZ/2.0);
		this.roi = roi;
			
		// Define discretization of accumulator array
		this.nPsi = nPsi;
		this.nPhi = nPhi;
		this.nRho = nRho;
		this.nOmega = nOmega;
		this.dPsi = (this.psiMax-this.psiMin)/(this.nPsi-1);
		this.dPhi = (this.phiMax-this.phiMin)/(this.nPhi-1);
		double[] diagonal = {this.dataSizeX, this.dataSizeY, this.dataSizeZ};
		this.rhoMax = this.magn(diagonal)/2.0;
		this.rhoMin = -this.rhoMax;
		this.dRho = (this.rhoMax-this.rhoMin)/(this.nRho-1);
		this.dOmega = (this.omegaMax-this.omegaMin)/(this.nOmega-1);
		this.accumulatorSizePsi = this.nPsi;
		this.accumulatorSizePhi = this.nPhi;
		this.accumulatorSizeRho = this.nRho;
		this.accumulatorSizeOmega = this.nOmega;
		this.accumulatorCenterRho = (int)(this.accumulatorSizeRho/2.0);
		this.accumulator = new int[this.accumulatorSizePsi][this.accumulatorSizePhi][this.accumulatorSizeRho][this.accumulatorSizeOmega];
	}
		
	public final void transform(boolean runBackmapping) {
		this.transformKernel(false);
		if (runBackmapping) {
			this.accumulatorBackmapped = new int[this.accumulatorSizePsi][this.accumulatorSizePhi][this.accumulatorSizeRho][this.accumulatorSizeOmega];
			this.transformKernel(true);
		}
	}
	
	private final void transformKernel(boolean isBackmappingRun) {
		// Consider ROI
		final int xMin = Math.max(0, this.roi[0][0]); int xMax = Math.min(this.roi[0][1], this.dataSizeX);
		final int yMin = Math.max(0, this.roi[1][0]); int yMax = Math.min(this.roi[1][1], this.dataSizeY);
		final int zMin = Math.max(0, this.roi[2][0]); int zMax = Math.min(this.roi[2][1], this.dataSizeZ);
		
		for (int z = zMin; z <= zMax; z++) {
			System.out.print(z+" ");
			for (int y = yMin; y <= yMax; y++) {
				for (int x = xMin; x <= xMax; x++) {
					
					if (this.data[z][y][x] > 0) {
						int[] point = {x - this.dataCenterX, y - this.dataCenterY, z - this.dataCenterZ};
						this.processPoint(point, isBackmappingRun);
					}
				}
			}
		}
	}
	
	public int[][][] getDataInRoi() {
		// ROI definition is inclusive
		int roiXlen = this.roi[0][1]-this.roi[0][0]+1;
		int roiYlen = this.roi[1][1]-this.roi[1][0]+1;
		int roiZlen = this.roi[2][1]-this.roi[2][0]+1;
		int[][][] dataInRoi = new int[roiZlen][roiYlen][roiXlen];
		for (int z=0; z < roiZlen; z++) {
			for (int y=0; y < roiYlen; y++) {
				for (int x=0; x < roiXlen; x++) {
					int dataX = this.roi[0][0] + x;
					int dataY = this.roi[1][0] + y;
					int dataZ = this.roi[2][0] + z;
					dataInRoi[z][y][x] = this.data[dataZ][dataY][dataX];
				}
			}
		}
		return dataInRoi;
	}
	
	public int[][][][] getAccumulator(boolean usedBackmapping) {
		if (usedBackmapping) {
			return this.accumulatorBackmapped;
		} else {
			return this.accumulator;
		}
	}
	
	private final void processPoint(int[] point, boolean isBackmappingRun) {
		// Allocate vectors only once
		double[] t = new double[3];
		double[] tBand = new double[3];
		double[] u = new double[3];
		double[] v = new double[3];
		
		// Backmapping variables
		int maxFrequency = 0;
		int maxPsiIndex = -1;
		int maxPhiIndex = -1;
		int maxRhoIndex = -1;
		int maxOmegaIndex = -1;
		
		for (int psiIndex = 0; psiIndex < this.accumulatorSizePsi; psiIndex++) {
			for (int phiIndex = 0; phiIndex < this.accumulatorSizePhi; phiIndex++) {
				
				// define trigonometric factors (for better performance)
				final double psi = psiIndex*this.dPsi;
				final double phi = phiIndex*this.dPhi;
				final double cosPhi = Math.cos(phi);
				final double sinPhi = Math.sin(phi);
				final double cosPsi = Math.cos(psi);
				final double sinPsi = Math.sin(psi);

				final double rho = (
					point[0]*sinPhi*cosPsi + point[1]*sinPhi*sinPsi + point[2]*cosPhi
				)/(
					Math.pow(sinPhi*cosPsi, 2) + Math.pow(sinPhi*sinPsi, 2) + Math.pow(cosPhi, 2)
				);
				final int rhoIndex = (int)(rho/this.dRho) + this.accumulatorCenterRho;
				
				// define point T and vector, tp, from T to P
				t[0] = rho*sinPhi*cosPsi;
				t[1] = rho*sinPhi*sinPsi;
				t[2] = rho*cosPhi;
				
				if (point[0]==t[0] && point[1]==t[1] && point[2]==t[2]) {
					// special (rare) case: P==T. all omega are candidates
					for (int omegaIndex=0; omegaIndex<this.accumulatorSizeOmega; omegaIndex++) {
						this.accumulator[psiIndex][phiIndex][rhoIndex][omegaIndex]++;
					}
					
				} else {
					// define tangent plane unit vectors u and v
					
					// define point U in the tangent plane (u = TU = OU-OT) 
					// U: (phi_u, psi, rho_u)
					final double phi_u = phi - this.dPhi;
					final double rho_u = rho/Math.cos(this.dPhi);
					u[0] = rho_u*Math.sin(phi_u)*cosPsi	- t[0];
					u[1] = rho_u*Math.sin(phi_u)*sinPsi	- t[1];
					u[2] = rho_u*Math.cos(phi_u)		- t[2];
						
					//	v = u x t
					v[0] = u[1]*t[2] - u[2]*t[1];
					v[1] = u[2]*t[0] - u[0]*t[2];
					v[2] = u[0]*t[1] - u[1]*t[0];
					
					if (isBackmappingRun) {
						final double omega = this.calculateOmega(t, point, u, v);
						final int omegaIndex = (int)(omega/this.dOmega);
						
						final int frequency = this.accumulator[psiIndex][phiIndex][rhoIndex][omegaIndex];
						if (frequency > maxFrequency) {
							maxFrequency = frequency;
							maxPsiIndex = psiIndex;
							maxPhiIndex = phiIndex;
							maxRhoIndex = rhoIndex;
							maxOmegaIndex = omegaIndex;
						}
						
					} else {
						// scalar product of t and [0 1 0] to find mu
						final double sty = t[1]/rho;
						double alpha = Math.acos(sty);
						if (alpha > Math.PI/2) {
							alpha -= Math.PI/2;
						}
						
						double mu = Math.abs(sty); // mu = |cos(alpha)|
						if (Math.PI/4 <= alpha && alpha <= 3*Math.PI/4) {
							mu = Math.sin(alpha);
						}
											
						final double rhoLower = rho - mu/2.0;
						final double rhoUpper = rhoLower + mu;
						int rhoLowerIndex = (int) Math.ceil(rhoLower/this.dRho) + this.accumulatorCenterRho;
						int rhoUpperIndex = (int) Math.ceil(rhoUpper/this.dRho -1) + this.accumulatorCenterRho;
						
						for (int rhoBandIndex = rhoLowerIndex; rhoBandIndex <= rhoUpperIndex; rhoBandIndex++) {
							final double rhoBand = (rhoBandIndex-this.accumulatorCenterRho)*this.dRho;
							tBand[0] = rhoBand*sinPhi*cosPsi;
							tBand[1] = rhoBand*sinPhi*sinPsi;
							tBand[2] = rhoBand*cosPhi;
							final double omegaBand = this.calculateOmega(tBand, point, u, v);
							final int omegaBandIndex = (int)(omegaBand/this.dOmega);
							
							this.accumulator[psiIndex][phiIndex][rhoBandIndex][omegaBandIndex]++;
						}
					}
				}
			}
		}
		if (isBackmappingRun) {
			if (maxPsiIndex>0 && maxPhiIndex>0 && maxRhoIndex>0 && maxOmegaIndex>0) {
				this.accumulatorBackmapped[maxPsiIndex][maxPhiIndex][maxRhoIndex][maxOmegaIndex]++;
			}
		}
	}
	
	/*
	 * Finds the polar coordinate angle of the point p in the coordinate system (u,v) 
	 */
	private final double calculateOmega(double[] t, int[] p, double[] u, double[] v) {
		double[] tp = new double[3];
		tp[0] = p[0]-t[0];
		tp[1] = p[1]-t[1];
		tp[2] = p[2]-t[2];

		// compute scalar products of tp versus u and v 
		final double su = u[0]*tp[0] + u[1]*tp[1] + u[2]*tp[2];
		final double sv = v[0]*tp[0] + v[1]*tp[1] + v[2]*tp[2];
		
		// compute vector magnitudes for use in the angle-scalar product relation 
		final double tpBandMagn = this.magn(tp);
		final double vMagn = this.magn(v);
		//System.out.println(tpMagn+", "+vMagn);
		
		final double omegaBand = Math.acos( (Math.signum(su)*sv)/(vMagn*tpBandMagn) );
		return omegaBand;
	}
	
	/*
	 * Compute the magnitude of the input vector, v.
	 */
	private final double magn(double[] vector) {
		double magnitudeSquared = 0;
		for (double component : vector) {
			magnitudeSquared += Math.pow(component, 2);
		}
		return Math.sqrt(magnitudeSquared);
	}
	
	public void writeNormalizedAccumulator(String filename, int maxValue, int maxNormValue) {
		File file = new File (filename);
		int numFgPoints = 0;

		// Now write the data array to the file.
	    try {
			// Create an output stream to the file.
			FileOutputStream file_output = new FileOutputStream (file);
			// Wrap the FileOutputStream with a DataOutputStream
			DataOutputStream data_out = new DataOutputStream (file_output);
			byte[] bytes = new byte[this.accumulatorSizeOmega];

			// Write the data to the file in an integer/double pair
			for (int i=0; i < this.accumulatorSizePsi; i++) {
				for (int j=0; j < this.accumulatorSizePhi; j++) {
				//int j = 10;
					for (int k=0; k < this.accumulatorSizeRho; k++) {
						for (int l=0; l < this.accumulatorSizeOmega; l++) {
							int normalizedValue = (int) (maxNormValue * (double)(this.accumulator[i][j][k][l])/((double)maxValue));
							//System.out.println((byte)normalizedValue);
							//data_out.writeByte(normalizedValue);
							bytes[l] = (byte) normalizedValue;
						}
						data_out.write(bytes, 0, bytes.length);
					}
				}
			}
			// Close file when finished with it..
			file_output.close ();
			System.out.println("File output complete.");
			System.out.println("Bytes written: "+data_out.size());
	    }
	    catch (IOException e) {
	       System.out.println ("IO exception = " + e );
	    }
	    
	}
	
	public double[][] getMaximaByThreshold(int threshold, boolean usedBackmapping) {
		if (threshold == 0) {
			return new double[0][0];
		}
		int[][][][] acc;
		if (usedBackmapping) {
			acc = this.accumulatorBackmapped;
		} else {
			acc = this.accumulator;
		}
		ArrayList<ArrayList<Double>> paramSets = new ArrayList<ArrayList<Double>>(); 
		for (int psiIndex = 0; psiIndex < this.accumulatorSizePsi; psiIndex++) {
			for (int phiIndex = 0; phiIndex < this.accumulatorSizePhi; phiIndex++) {
				for (int rhoIndex = 0; rhoIndex < this.accumulatorSizeRho; rhoIndex++) {
					for (int omegaIndex = 0; omegaIndex < this.accumulatorSizeOmega; omegaIndex++) {
						
						if (acc[psiIndex][phiIndex][rhoIndex][omegaIndex] >= threshold) {
							ArrayList<Double> paramSet = new ArrayList<Double>();
							paramSet.add(psiIndex*this.dPsi);
							paramSet.add(phiIndex*this.dPhi);
							paramSet.add((rhoIndex-this.accumulatorCenterRho)*this.dRho);
							paramSet.add(omegaIndex*this.dOmega);
							paramSets.add(paramSet);
						}
					}
				}
			}
		}
		double[][] result = new double[paramSets.size()][4]; 
		for (int i=0; i<paramSets.size(); i++) {
			ArrayList<Double> paramSet = paramSets.get(i); 
			for (int j=0; j<paramSet.size(); j++) {
				result[i][j] = (double)paramSet.get(j);
			}
		}
		return result;
	}
	
	public double[][] getMaximaByNumber(int numberOfPoints, boolean usedBackmapping) {
		int[][][][] acc;
		if (usedBackmapping) {
			acc = this.accumulatorBackmapped;
		} else {
			acc = this.accumulator;
		}
		TreeMap<HoughAccPoint, Integer> sortedAcc = new TreeMap<HoughAccPoint, Integer>();
		for (int psiIndex = 0; psiIndex < this.accumulatorSizePsi; psiIndex++) {
			for (int phiIndex = 0; phiIndex < this.accumulatorSizePhi; phiIndex++) {
				for (int rhoIndex = 0; rhoIndex < this.accumulatorSizeRho; rhoIndex++) {
					for (int omegaIndex = 0; omegaIndex < this.accumulatorSizeOmega; omegaIndex++) {
						int frequency = acc[psiIndex][phiIndex][rhoIndex][omegaIndex];
						/*
						if (rhoIndex == this.accumulatorCenterRho) {
							continue;
						}*/
						if (frequency > 0) {
							sortedAcc.put(new HoughAccPoint(psiIndex, phiIndex, rhoIndex, omegaIndex), frequency);
						}
					}
				}
			}
		}
		/*
		List<Map.Entry<Integer, HoughAccPoint>> list = new Vector<Map.Entry<Integer, HoughAccPoint>>(sortedAcc.entrySet());
		
		*/
		List<Map.Entry<HoughAccPoint, Integer>> list = new Vector<Map.Entry<HoughAccPoint, Integer>>(sortedAcc.entrySet());
		/*
		for (Map.Entry<HoughAccPoint, Integer> entry : list) {
			System.out.println(entry.getKey()+": "+entry.getValue());
		}
		*/
		Collections.sort(list, new Comparator<Map.Entry<HoughAccPoint, Integer>>(){
			public int compare(Map.Entry<HoughAccPoint, Integer> entry, Map.Entry<HoughAccPoint, Integer> entry1) {
				return (entry.getValue().equals(entry1.getValue()) ? 0 : (entry.getValue() > entry1.getValue() ? -1 : 1));
			}
		});
		
		double[][] maxima = new double[Math.min(numberOfPoints, list.size())][4];
		for (int pointCounter = 0; pointCounter < maxima.length; pointCounter++) {
			HoughAccPoint point = list.get(pointCounter).getKey();
			maxima[pointCounter][0] = this.dPsi*point.psiIndex;
			maxima[pointCounter][1] = this.dPhi*point.phiIndex;
			maxima[pointCounter][2] = this.dRho*(point.rhoIndex-this.accumulatorCenterRho);
			maxima[pointCounter][3] = this.dOmega*point.omegaIndex;
		}
		return maxima;
	}
	
	public int getHighestFrequency(boolean usedBackmapping) {
		int[][][][] acc;
		if (usedBackmapping) {
			acc = this.accumulatorBackmapped;
		} else {
			acc = this.accumulator;
		}
		
		// Find highest frequency
		int highestFrequency = 0;
		for (int psiIndex = 0; psiIndex < this.accumulatorSizePsi; psiIndex++) {
			for (int phiIndex = 0; phiIndex < this.accumulatorSizePhi; phiIndex++) {
				for (int rhoIndex = 0; rhoIndex < this.accumulatorSizeRho; rhoIndex++) {
					for (int omegaIndex = 0; omegaIndex < this.accumulatorSizeOmega; omegaIndex++) {
						
						highestFrequency = Math.max(highestFrequency, acc[psiIndex][phiIndex][rhoIndex][omegaIndex]);
					}
				}
			}
		}
		return highestFrequency;
	}
	
	public void showAccStats(boolean usedBackmapping) {
		int highestFrequency = this.getHighestFrequency(usedBackmapping);
		int[][][][] acc;
		if (usedBackmapping) {
			acc = this.accumulatorBackmapped;
		} else {
			acc = this.accumulator;
		}
		
		// Collect frequencies
		int[] frequencyFrequencies = new int[highestFrequency+1]; // frequencies of frequencies
		for (int psiIndex = 0; psiIndex < this.accumulatorSizePsi; psiIndex++) {
			for (int phiIndex = 0; phiIndex < this.accumulatorSizePhi; phiIndex++) {
				for (int rhoIndex = 0; rhoIndex < this.accumulatorSizeRho; rhoIndex++) {
					for (int omegaIndex = 0; omegaIndex < this.accumulatorSizeOmega; omegaIndex++) {
						
						int frequency = acc[psiIndex][phiIndex][rhoIndex][omegaIndex];
						frequencyFrequencies[frequency]++;
					}
				}
			}
		}
		
		// Print results
		System.out.println("Frequency count:");
		for (int accValue=0; accValue<frequencyFrequencies.length; accValue++) {
			int frequency = frequencyFrequencies[accValue];
			System.out.println("  "+accValue+": "+frequency);
		}
		System.out.println("\n");
		System.out.println("Highest frequencies:");
		
		for (int psiIndex = 0; psiIndex < this.accumulatorSizePsi; psiIndex++) {
			for (int phiIndex = 0; phiIndex <  this.accumulatorSizePhi; phiIndex++) {
				for (int rhoIndex = 0; rhoIndex < this.accumulatorSizeRho; rhoIndex++) {
					for (int omegaIndex = 0; omegaIndex < this.accumulatorSizeOmega; omegaIndex++) {
						
						int accValue = acc[psiIndex][phiIndex][rhoIndex][omegaIndex];
						if (accValue >= highestFrequency) {
							double psi = psiIndex*this.dPsi;
							double phi = phiIndex*this.dPhi;
							double rho = (rhoIndex-this.accumulatorCenterRho)*this.dRho;
							double omega = omegaIndex*this.dOmega;
							System.out.println("  psi="+psi+", phi="+phi+", rho="+rho+", omega="+omega);
						}
					}
				}
			}
		}
		
	}
	
	public static int[][][] readImage(String filename, int x, int y, int z) {
		File file = new File (filename);
		int[][][] image = new int[z][y][x];

	    try {
			// Wrap the FileInputStream with a DataInputStream
			FileInputStream file_input = new FileInputStream (file);
			DataInputStream data_in    = new DataInputStream (file_input );
			System.out.println("Started reading file "+filename);

			for (int zIndex=0; zIndex < image.length; zIndex++) {
				System.out.print(zIndex+" ");
        		for (int yIndex=0; yIndex < image[zIndex].length; yIndex++) {
        			byte[] xBytes = new byte[image[zIndex][yIndex].length];
        			data_in.read(xBytes); 
        			for (int xIndex=0; xIndex < image[zIndex][yIndex].length; xIndex++) {
		        		image[zIndex][yIndex][xIndex] = (int) xBytes[xIndex];
		        	}
        		}
			}
			System.out.println("");
			System.out.println("Read "+x+"-"+y+"-"+z+" image from file "+filename);
			data_in.close ();
			
	    } catch (EOFException eof) {
			System.err.println ("End of File");
			
        } catch  (IOException e) {
	       System.err.println ( "IO Exception =: " + e );
	    }
        
	    return image;
	}
	
	
	public static void main(String[] args) {
		
		int[][][] image = new int[30][20][20];
		for(int z = 0; z < image.length; z++) {
			for (int y = 0;  y <image[z].length; y++) {
				for (int x = 0; x < image[z][y].length; x++) {
					image[z][y][x] = 0;
				}
			}
		}
		
		image[19][19][19] = 1;
		image[18][19][18] = 1;
		image[17][19][17] = 1;
		image[16][19][16] = 1;
		image[15][19][15] = 1;
		image[14][19][14] = 1;
		image[13][19][13] = 1;
		image[12][19][12] = 1;
		image[11][19][11] = 1;
		image[10][19][10] = 1;
		
		image[4][0][0] = 1;
		image[4][0][1] = 1;
		image[4][0][2] = 1;
		image[4][0][3] = 1;
		image[4][0][4] = 1;
		image[4][0][5] = 1;
		image[4][0][6] = 1;
		image[4][0][7] = 1;
		image[4][0][8] = 1;
		image[4][0][9] = 1;
		
		image[28][0][0] = 1;
		image[28][0][1] = 1;
		image[28][0][2] = 1;
		image[28][0][3] = 1;
		image[28][0][4] = 1;
		image[28][0][5] = 1;
		image[28][0][6] = 1;
		image[28][0][7] = 1;
		image[28][0][8] = 1;
		image[28][0][9] = 1;
		image[28][0][10] = 1;
		image[28][0][11] = 1;
		image[28][0][12] = 1;
		image[28][0][13] = 1;
		image[28][0][14] = 1;
		image[28][0][15] = 1;
		image[28][0][16] = 1;
		image[28][0][17] = 1;
		image[28][0][18] = 1;
		image[28][0][19] = 1;
		
		//int[][][] image = HoughTransformation.readImage("input.img", 256, 517, 256);
		
		int[][] roi = new int[3][2];


		// inclusive!
		/*
		
		roi[0][0] = 30; roi[0][1] = 110; // xmin, xmax
		roi[1][0] = 50; roi[1][1] = 120; // ymin, ymax
		roi[2][0] = 170; roi[2][1] = 210; // zmin, zmax
		 */
		roi[0][0] = 0; roi[0][1] = 19; // xmin, xmax
		roi[1][0] = 0; roi[1][1] = 19; // ymin, ymax
		roi[2][0] = 0; roi[2][1] = 29; // zmin, zmax

		HoughTransformation hough = new HoughTransformation(image, 30, 30, 60, 30, roi);
		System.out.println("Hough transformation started");
		boolean useBackmapping = true;
		hough.transform(useBackmapping);
		System.out.println("");
		System.out.println("Hough transformation completed");
		
		//hough.showAccStats(useBackmapping);
		
		int highestFrequency = hough.getHighestFrequency(useBackmapping);
		System.out.println("Highest frequency: "+highestFrequency);
		//double[][] maxima = hough.getMaximaByThreshold(highestFrequency, useBackmapping);
		double[][] maxima = hough.getMaximaByNumber(500, useBackmapping);
		
		/*
		System.out.println("Results: ");
		for (double[] maximum : maxima) {
			for (double parameter : maximum) {
				System.out.print(parameter);
				System.out.print("\t");
			}
			System.out.print("\n");
		}*/
		
		
		// Visualize
		
		HoughDisplayControl plot = new HoughDisplayControl();
		plot.renderer.setLines(maxima);
		plot.renderer.setImage(hough.getDataInRoi());
		plot.renderer.setParameterImage(hough.getAccumulator(useBackmapping), highestFrequency);
		
		Thread displayT = new Thread(plot);
		HoughDisplayControl.displayT = displayT;
		HoughDisplayControl.displayT.start();

	
		// Write to file
		
		//hough.writeNormalizedAccumulator("output.img", highestFrequency, 127);
		
	}
	
}
 